using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

namespace Frets.Game {
	partial class Game {

		
		Matrix viewMatrix;
		Matrix projectionMatrix;
		BasicEffect basicEffect;

		SpriteBatch ForegroundBatch;
		SpriteFont MainFont;

		Model FretButton;
		Model Sustain;

		VertexDeclaration vertexDeclaration;

		VertexPositionTexture[] FretboardPoints;
		short[] FretboardListIndices;

		int NoteProgress = 0;

		public void LoadContent(ContentManager content, GraphicsDeviceManager graphics) {


			vertexDeclaration = new VertexDeclaration(
				graphics.GraphicsDevice,
				VertexPositionNormalTexture.VertexElements);

			basicEffect = new BasicEffect(graphics.GraphicsDevice, null);
			basicEffect.DiffuseColor = new Vector3(1.0f, 1.0f, 1.0f);

			basicEffect.View = viewMatrix;
			basicEffect.Projection = projectionMatrix;
			
			this.FretboardPoints = new VertexPositionTexture[] {
					new VertexPositionTexture(new Vector3(-5f, 0f, -200f), Vector2.Zero),
					new VertexPositionTexture(new Vector3(+5f, 0f, -200f), Vector2.Zero),
					new VertexPositionTexture(new Vector3(+5f, 0f,  +10f), Vector2.Zero),
					new VertexPositionTexture(new Vector3(-5f, 0f,  +10f), Vector2.Zero),
				};
			this.FretboardListIndices = new short[] {
					0, 1, 2,
					2, 3, 0
				};

			viewMatrix = Matrix.CreateLookAt(
				new Vector3(0f, 10f,  15f),
				new Vector3(0f,  0f, -15f),
				Vector3.Up
				);

			projectionMatrix = Matrix.CreatePerspectiveFieldOfView(
				MathHelper.ToRadians(45),
				(float)graphics.GraphicsDevice.Viewport.Width /
				(float)graphics.GraphicsDevice.Viewport.Height,
				0.1f, 500.0f);

			this.MainFont = content.Load<SpriteFont>("Content/Fonts/MainFont");
			this.FretButton = content.Load<Model>("Content/Models/FretButton");
			this.Sustain = content.Load<Model>("Content/Models/Sustain");
			this.ForegroundBatch = new SpriteBatch(graphics.GraphicsDevice);



		}

		public void Draw(GraphicsDeviceManager graphics) {

			graphics.GraphicsDevice.RenderState.DepthBufferEnable = true;
			graphics.GraphicsDevice.VertexDeclaration = vertexDeclaration;
			graphics.GraphicsDevice.Clear(Color.Black);

			TimeSpan T = this.CurrentSong.Position - TimeSpan.FromSeconds(5d);
			while (NoteProgress < this.CurrentSong.FretEvents.Length && (this.CurrentSong.FretEvents[NoteProgress].StartTime + this.CurrentSong.FretEvents[NoteProgress].Duration) < T) {
				++NoteProgress;
			}



			basicEffect.View = this.viewMatrix;
			basicEffect.Projection = this.projectionMatrix;
			basicEffect.World = Matrix.CreateTranslation(Vector3.Zero);
		
			basicEffect.Begin();
			foreach (EffectPass pass in basicEffect.CurrentTechnique.Passes) {
				pass.Begin();


				// Draw fret board:
				graphics.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionTexture>(PrimitiveType.TriangleList, FretboardPoints, 0, 6, FretboardListIndices, 0, 2);




				// Draw fret buttons:

				Stack<Song.Notes.FretEvent> FretButtonsToDraw = new Stack<Song.Notes.FretEvent>();

				int StartFret = this.NoteProgress;
				TimeSpan CurrentTime = this.currentSong.Position;
				while (StartFret < this.CurrentSong.FretEvents.Length) {

					Song.Notes.FretEvent E = this.CurrentSong.FretEvents[StartFret];
					for (int i = 0; i < 5; ++i) {
						Song.Notes.HeldFrets TestFret = (Song.Notes.HeldFrets)(1 << i);
						if ((E.Frets & TestFret) != Song.Notes.HeldFrets.None) {
							Song.Notes.FretEvent F = E;
							F.Frets = TestFret;
							F.StartTime = E.StartTime - CurrentTime;
							FretButtonsToDraw.Push(F);
						}
					}
					++StartFret;
					if ((E.StartTime - CurrentTime) > TimeSpan.FromSeconds(5f)) break;
				}

				while (FretButtonsToDraw.Count > 0) {
					this.DrawFretButton(FretButtonsToDraw.Pop(), Vector3.Zero);
				}

				// Draw user-input buttons (last for alpha reasons).
				Vector3 Floating = new Vector3(0f, 1f, 0f);

				Song.Notes.FretEvent DrawKeyboardButtons = new Song.Notes.FretEvent();

				bool[] FretsPressed = new bool[] { Fret0Pressed, Fret1Pressed, Fret2Pressed, Fret3Pressed, Fret4Pressed };

				for (int i = 0; i < FretsPressed.Length; ++i) {
					DrawKeyboardButtons.Frets = (Song.Notes.HeldFrets)(1 << i);
					DrawKeyboardButtons.NoteHit = FretsPressed[i] && this.HitLastNoteSuccessfully && (DrawKeyboardButtons.Frets & this.ExpectedFrets) != Song.Notes.HeldFrets.None;
					DrawFretButton(DrawKeyboardButtons, !FretsPressed[i] ? Floating : Vector3.Zero);	
				}
				
				pass.End();
			}
			basicEffect.End();

			string output = this.CurrentSong == null ? "" : this.CurrentSong.Name + Environment.NewLine + this.CurrentSong.Artist;


			ForegroundBatch.Begin();
			ForegroundBatch.DrawString(this.MainFont, output, new Vector2(10f, 0f), Color.White, 0.0f, Vector2.Zero, 0.5f, SpriteEffects.None, 0.5f);
			ForegroundBatch.End();

		}


		private Random RandomSource = new Random();

		private void DrawFretButton(Song.Notes.FretEvent fret, Vector3 offset) {

			float SpeedScaler = fret.NoteSpeed;

			Vector3 Position = Vector3.Zero;

			Vector3 FretColour = Vector3.Zero;


			switch (fret.Frets) {
				case Song.Notes.HeldFrets.Fret0:
					Position.X = -2f;
					FretColour = Color.LightGreen.ToVector3();
					break;
				case Song.Notes.HeldFrets.Fret1:
					Position.X = -1f;
					FretColour = Color.Red.ToVector3();
					break;
				case Song.Notes.HeldFrets.Fret2:
					Position.X =  0f;
					FretColour = Color.Gold.ToVector3();
					break;
				case Song.Notes.HeldFrets.Fret3:
					Position.X = +1f;
					FretColour = Color.SkyBlue.ToVector3();
					break;
				case Song.Notes.HeldFrets.Fret4:
					Position.X = +2f;
					FretColour = Color.DarkOrange.ToVector3();
					break;
			}

			if (!fret.NoteHit) {
				if (fret.StartTime < TimeSpan.Zero) {
					FretColour = Color.Gray.ToVector3();
				} else {
					FretColour *= 0.5f;
				}
			}

			Position.X *= 2f;

			Position.Z = (float)fret.StartTime.TotalSeconds * -SpeedScaler;

			Position += offset;

			

			// Draw tails:
			if (fret.Duration > TimeSpan.Zero) {
				foreach (ModelMesh mesh in this.Sustain.Meshes) {
					foreach (BasicEffect effect in mesh.Effects) {
						effect.EnableDefaultLighting();
						effect.DiffuseColor = FretColour;
						effect.View = this.viewMatrix;
						effect.Projection = this.projectionMatrix;
						effect.World = Matrix.CreateScale(new Vector3(1f, 1f, (float)fret.Duration.TotalSeconds * SpeedScaler)) * Matrix.CreateTranslation(Position);
						effect.Alpha = 1f;
					}
					mesh.Draw();

					if (fret.NoteHit) {
						for (int i = 0; i < 3; ++i) {
							foreach (BasicEffect effect in mesh.Effects) {
								effect.EnableDefaultLighting();
								effect.DiffuseColor = FretColour;
								effect.View = this.viewMatrix;
								effect.Projection = this.projectionMatrix;
								effect.World = Matrix.CreateScale(new Vector3(1f, 1f, (float)fret.Duration.TotalSeconds * SpeedScaler)) * Matrix.CreateTranslation(Position + 0.2f * new Vector3((0.5f - (float)RandomSource.NextDouble()), (0.5f - (float)RandomSource.NextDouble()), (0.5f - (float)RandomSource.NextDouble())));
								effect.Alpha = 0.5f;
							}
							mesh.Draw();
						}
					}

				}
			}

			// Draw button:
			foreach (ModelMesh mesh in this.FretButton.Meshes) {

				foreach (BasicEffect effect in mesh.Effects) {
					effect.EnableDefaultLighting();
					effect.DiffuseColor = FretColour;
					effect.View = this.viewMatrix;
					effect.Projection = this.projectionMatrix;
					effect.World = Matrix.CreateTranslation(Position);
					effect.Alpha = 1f;
				}
				mesh.Draw();

				if (fret.NoteHit) {
					for (int i = 0; i < 3; ++i) {
						foreach (BasicEffect effect in mesh.Effects) {
							effect.EnableDefaultLighting();
							effect.DiffuseColor = FretColour;
							effect.View = this.viewMatrix;
							effect.Projection = this.projectionMatrix;
							effect.World = Matrix.CreateScale(1f + 0.4f * (float)RandomSource.NextDouble()) * Matrix.CreateTranslation(Position + 0.2f * new Vector3((0.5f - (float)RandomSource.NextDouble()), (0.5f - (float)RandomSource.NextDouble()), (0.5f - (float)RandomSource.NextDouble())));
							effect.Alpha = 0.5f;
						}
						mesh.Draw();
					}
				}
			}
		}
	}
}
