using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Frets.Game {
	partial class Song {

		/// <summary>
		/// Class that handles playback of an in-game song.
		/// </summary>
		class Player {

			/// <summary>
			/// The FMOD sound system we're playing on.
			/// </summary>
			private readonly FmodEx.Fmod SoundSystem;

			/// <summary>
			/// Sound of the guitar line.
			/// </summary>
			private readonly FmodEx.Fmod.Sound SoundGuitar;

			/// <summary>
			/// Sound of the backing track.
			/// </summary>
			private readonly FmodEx.Fmod.Sound SoundSong;

			/// <summary>
			/// Create an instance of the Player class.
			/// </summary>
			/// <param name="soundSystem">FMOD sound system to play back on.</param>
			/// <param name="songPath">Directory of the song file.</param>
			public Player(FmodEx.Fmod soundSystem, string songPath) {
				this.SoundSystem = soundSystem;
				this.SoundGuitar = new FmodEx.Fmod.Sound(this.SoundSystem, Path.Combine(songPath, "guitar.ogg"), true);
				this.SoundSong = new FmodEx.Fmod.Sound(this.SoundSystem, Path.Combine(songPath, "song.ogg"), true);
			}

			/// <summary>
			/// Start playing both tracks.
			/// </summary>
			/// <remarks></remarks>
			public void Play() {
				this.SoundGuitar.Play(true);
				this.SoundSong.Play(true);
				this.GuitarPlaying = false;
				this.SoundGuitar.Paused = false;
				this.SoundSong.Paused = false;
			}

			public bool GuitarPlaying {
				set { this.SoundGuitar.Mute(!value); }
				get { return !this.SoundGuitar.IsMuted; }
			}

			public TimeSpan Position {
				get { return this.SoundGuitar.Position; }
			}
		}
	}
}
