using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using System.IO;

namespace Frets.Game {
	partial class Song {


		private readonly FmodEx.Fmod SoundSystem;

		private readonly string BaseDirectory;

		private Player SongPlayer;

		private Notes SongNotes;
		public Notes.DifficultyModes AvailableDifficulties {
			get { return this.SongNotes.AvailableModes; }
		}

		#region Properties

		private Color casetteColour = Color.Yellow;
		/// <summary>
		///  Colour of the cassette.
		/// </summary>
		public Color CasetteColour {
			get { return this.casetteColour; }
		}

		private string name = "Untitled";
		/// <summary>
		///  Name of the song.
		/// </summary>
		public string Name {
			get { return this.name; }
		}

		private string artist = "Unknown Artist";
		/// <summary>
		///  Name of the artist.
		/// </summary>
		public string Artist {
			get { return this.artist; }
		}

		public TimeSpan Position {
			get { return this.SongPlayer.Position; }
		}


		#endregion

		public Notes.HeldFrets GetExpectedFrets(TimeSpan time) {
			foreach (Notes.FretEvent FE in this.FretEvents) {
				if (time >= FE.StartTime && time <= FE.EndTime) return FE.Frets;
			}
			return Notes.HeldFrets.None;
		}

		/// <summary>
		/// Create a new song instance.
		/// </summary>
		/// <param name="baseDirectory">The directory to load the song data from.</param>
		public Song(FmodEx.Fmod soundSystem, string baseDirectory) {

			this.SoundSystem = soundSystem;
			this.BaseDirectory = baseDirectory;

			// Load the song information:
			Parsers.IniFile IniFile = new Parsers.IniFile(Path.Combine(this.BaseDirectory, "song.ini"));
			if (IniFile.Sections.ContainsKey("song")) {
				Dictionary<string, string> SongInfo = IniFile.Sections["song"];
				foreach (KeyValuePair<string, string> SongInfoField in SongInfo) {
					switch (SongInfoField.Key) {
						case "cassettecolor":
							string Hex = SongInfoField.Value;
							if (Hex.StartsWith("#")) {
								int ColourCode = Convert.ToInt32(Hex.Substring(1), 16);
								this.casetteColour = new Color((byte)ColourCode, (byte)(ColourCode >> 8), (byte)(ColourCode >> 16), 0xFF);
							}
							break;
						case "name":
							this.name = SongInfoField.Value;
							break;
						case "artist":
							this.artist = SongInfoField.Value;
							break;
					}
				}
			}

		}

		private Notes.FretEvent[] fretEvents;
		public Notes.FretEvent[] FretEvents {
			get { return this.fretEvents; }
		}

		public void LoadNotes() {
			// Decode the MIDI file:
			this.SongNotes = new Notes(Path.Combine(this.BaseDirectory, "notes.mid"));
		}
		public void LoadSound() {
			// Load the music:
			this.SongPlayer = new Player(this.SoundSystem, this.BaseDirectory);
		}

		public void Start(Notes.DifficultyModes difficulty) {
			this.fretEvents = this.SongNotes.GetSongFretEvents(difficulty);
			this.SongPlayer.Play();
		}

		public bool GuitarPlaying {
			set { this.SongPlayer.GuitarPlaying = value; }
			get { return this.SongPlayer.GuitarPlaying; }
		}

	}
}