using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Frets.Parsers {
	class IniFile {

		private readonly Dictionary<string, Dictionary<string, string>> sections;
		public Dictionary<string, Dictionary<string, string>> Sections {
			get { return this.sections; }
		}

		public IniFile(string filename) {
			string[] data = File.ReadAllLines(filename);
			this.sections = new Dictionary<string, Dictionary<string, string>>();

			Dictionary<string, string> CurrentSection = new Dictionary<string, string>();
			this.sections.Add("", CurrentSection);


			foreach (string s in data) {
				string BrokenLine = s.Split(';')[0].Trim();
				if (BrokenLine == string.Empty) continue;
				if (BrokenLine.StartsWith("[") && BrokenLine.EndsWith("]")) {
					string NewSection = BrokenLine.Substring(1, BrokenLine.Length - 2).ToLowerInvariant();
					if (!this.sections.TryGetValue(NewSection, out CurrentSection)) {
						CurrentSection = new Dictionary<string, string>();
						this.sections.Add(NewSection, CurrentSection);
					}
				} else {
					string[] IniPair = BrokenLine.Split('=');
					if (IniPair.Length < 2) throw new InvalidDataException("Expected key/value pair.");
					string Key = IniPair[0].Trim().ToLowerInvariant();
					if (CurrentSection.ContainsKey(Key)) CurrentSection.Remove(Key);
					IniPair[0] = string.Empty;
					string Value = string.Join("=", IniPair).Substring(1).Trim();
					CurrentSection.Add(Key, Value);
				}

			}
		}
	}
}
